@@ -23,24 +23,6 @@
 #include "param.h"
 
 
-/**
-   @brief Key for translating terminal vector.
- */
-class TermKey {
- public:
-  unsigned int base;
-  unsigned int extent;
-  unsigned int ptId;
-
-  inline void Init(unsigned int _base, unsigned int _extent, unsigned int _ptId) {
-    base = _base;
-    extent = _extent;
-    ptId = _ptId;
-  }
-};
-
-
-
 /**
  @brief Serialized representation of the pre-tree, suitable for tranfer between
  devices such as coprocessors, disks and nodes.
@@ -53,19 +35,45 @@ class TermKey {
 */
 class PTNode {
  public:
-  unsigned int id;
-  unsigned int lhId;  // LH subnode index. Nonzero iff non-terminal.
-  unsigned int predIdx; // Split only.
+  unsigned int lhDel;  // Delta to LH subnode. Nonzero iff non-terminal.
+  unsigned int predIdx; // Nonterminal only.
+  FltVal info; // Nonterminal only.
   union {
     unsigned int offset; // Bit-vector offset:  factor.
-    RankRange rankRange;//double rkMean; // Mean rank:  numeric.
+    RankRange rankRange; // LH, RH ranks:  numeric.
   } splitVal;
-  void Consume(const class PMTrain *pmTrain, class ForestTrain *forest, unsigned int tIdx);
+
+  void NonterminalConsume(const class PMTrain *pmTrain, class ForestTrain *forest, unsigned int tIdx, std::vector<double> &predInfo, unsigned int idx) const;
+
+
+  inline void SetTerminal() {
+    lhDel = 0;
+  }
+
+
+  inline void SetNonterminal(unsigned int parId, unsigned int lhId) {
+    lhDel = lhId - parId;
+  }
+
+  
+  inline bool NonTerminal() const {
+    return lhDel != 0;
+  }
+
+
+  inline unsigned int LHId(unsigned int ptId) const {
+    return NonTerminal() ? ptId + lhDel : 0;
+  }
+
+  inline unsigned int RHId(unsigned int ptId) const {
+    return NonTerminal() ? LHId(ptId) + 1 : 0;
+  }
 };
 
 
 class PreTree {
   static unsigned int heightEst;
+  static unsigned int leafMax; // User option:  maximum # leaves, if > 0.
   const class PMTrain *pmTrain;
   PTNode *nodeVec; // Vector of tree nodes.
   unsigned int nodeCount; // Allocation height of node vector.
@@ -73,59 +81,61 @@ class PreTree {
   unsigned int leafCount;
   unsigned int bitEnd; // Next free slot in factor bit vector.
   class BV *splitBits;
-  std::vector<TermKey> termKey;
   std::vector<unsigned int> termST;
   class BV *BitFactory();
   void TerminalOffspring(unsigned int _parId);
-  const std::vector<unsigned int> FrontierToLeaf(class ForestTrain *forest, unsigned int tIdx);
+  const std::vector<unsigned int> FrontierConsume(class ForestTrain *forest, unsigned int tIdx) const ;
   const unsigned int bagCount;
-  std::vector<double> info; // Aggregates info value of nonterminals, by predictor.
   unsigned int BitWidth();
 
  public:
   PreTree(const class PMTrain *_pmTrain, unsigned int _bagCount);
   ~PreTree();
-  static void Immutables(unsigned int _nSamp, unsigned int _minH);
+  static void Immutables(unsigned int _nSamp, unsigned int _minH, unsigned int _leafMax);
   static void DeImmutables();
   static void Reserve(unsigned int height);
 
-  const std::vector<unsigned int> DecTree(class ForestTrain *forest, unsigned int tIdx, std::vector<double> &predInfo);
-  void NodeConsume(class ForestTrain *forest, unsigned int tIdx);
+  const std::vector<unsigned int> Consume(class ForestTrain *forest, unsigned int tIdx, std::vector<double> &predInfo);
+  void NonterminalConsume(class ForestTrain *forest, unsigned int tIdx, std::vector<double> &predInfo) const;
   void BitConsume(unsigned int *outBits);
   void LHBit(int idx, unsigned int pos);
   void NonTerminalFac(double _info, unsigned int _predIdx, unsigned int _id);
   void NonTerminalNum(double _info, unsigned int _predIdx, RankRange _rankRange, unsigned int _id);
   void Level(unsigned int splitNext, unsigned int leafNext);
   void ReNodes();
-  void SubtreeFrontier(const std::vector<TermKey> &stKey, const std::vector<unsigned int> &stTerm);
-
+  void SubtreeFrontier(const std::vector<unsigned int> &stTerm);
+  unsigned int LeafMerge();
   
-  /**
-     @brief Height accessor.
-   */
-  inline unsigned int Height() {
-    return height;
-  }
-
-
   inline unsigned int LHId(unsigned int ptId) const {
-    return nodeVec[ptId].lhId;
+    return nodeVec[ptId].LHId(ptId);
   }
 
   
   inline unsigned int RHId(unsigned int ptId) const {
-    unsigned int lhId = nodeVec[ptId].lhId;
-    return lhId != 0 ? lhId + 1 : 0;
+    return nodeVec[ptId].RHId(ptId);
   }
 
   
   /**
      @return true iff node is nonterminal.
    */
-  inline bool NonTerminal(unsigned int ptId) {
-    return nodeVec[ptId].lhId > 0;
+  inline bool NonTerminal(unsigned int ptId) const {
+    return nodeVec[ptId].NonTerminal();
   }
 
+
+    /**
+       @brief Determines whether a nonterminal can be merged with its
+       children.
+
+       @param ptId is the index of a nonterminal.
+
+       @return true iff node has two leaf children.
+    */
+  inline bool Mergeable(unsigned int ptId) const {
+    return !NonTerminal(LHId(ptId)) && !NonTerminal(RHId(ptId));
+  }  
+
   
   /**
      @brief Fills in references to values known to be useful for building
