@@ -20,6 +20,41 @@
 #include "param.h"
 
 
+/**
+   @brief Sum / count record for categorical indices.
+ */
+class SumCount {
+  double sum;
+  unsigned int sCount;
+
+ public:
+  void Init() {
+    sum = 0.0;
+    sCount = 0;
+  }
+
+  inline void Ref(double &_sum, unsigned int &_sCount) const {
+    _sum = sum;
+    _sCount = sCount;
+  }
+  
+  
+  inline void Accum(double _sum, unsigned int _sCount) {
+    sum += _sum;
+    sCount += _sCount;
+  }
+
+
+  /**
+     @brief Subtracts contents of vector passed.
+   */
+  void Decr(const SumCount &subtrahend) {
+    sum -= subtrahend.sum;
+    sCount -= subtrahend.sCount;
+  }
+};
+
+
 /**
    @brief Single node type for regression and classification.
 
@@ -41,10 +76,12 @@ class SampleNode {
 
  public:
 
-  inline void Set(FltVal _sum, unsigned int _sCount, unsigned int _ctg = 0) {
-    sum = _sum;
+  inline double Set(FltVal _yVal, unsigned int _sCount, unsigned int _ctg = 0) {
     sCount = _sCount;
+    sum = _yVal * sCount;
     ctg = _ctg;
+
+    return sum;
   }
 
   /**
@@ -81,37 +118,51 @@ class SampleNode {
 };
 
 
+
 /**
  @brief Run of instances of a given row obtained from sampling for an individual tree.
 */
 class Sample {
   class BV *treeBag;
   std::vector<unsigned int> row2Sample;
+
  protected:
+  const unsigned int nRow;
   const unsigned int noSample; // Inattainable sample index.
-  static unsigned int nRow;
   static unsigned int nSamp;
   std::vector<SampleNode> sampleNode;
+  std::vector<SumCount> ctgRoot;
   unsigned int bagCount;
   double bagSum;
-  class SamplePred *samplePred;
-  class Bottom *bottom;
-  unsigned int PreStage(const std::vector<double> &y, const std::vector<unsigned int> &yCtg, const class RowRank *rowRank, class SamplePred *&_samplePred);
-  void Stage(const class RowRank *rowRank);
-  void Stage(const class RowRank *rowRank, unsigned int predIdx);
-  void PackIndex(unsigned int row, unsigned int predRank, std::vector<class StagePack> &stagePack);
 
-  static void RowSample(std::vector<unsigned int> &sCountRow);
+  // Factories parametrized by coprocessor state.
+  static class SamplePred *SamplePredFactory(const class Coproc *coproc, unsigned int _nPred, unsigned int _bagCount, unsigned int _bufferSize);
+  static class SPCtg* SPCtgFactory(const class Coproc *coproc, const class PMTrain *pmTrain, const class RowRank *rowRank, unsigned int bagCount, unsigned int _nCtg);
+  static class SPReg* SPRegFactory(const class Coproc *coproc, const class PMTrain *pmTrain, const class RowRank *rowRank, unsigned int bagCount);
+  static unsigned int RowSample(std::vector<unsigned int> &sCountRow);
+  
+  
+  void PreStage(const std::vector<double> &y, const std::vector<unsigned int> &yCtg, const class RowRank *rowRank);
+  void Stage(const class RowRank *rowRank, class SamplePred *samplePred, class Bottom *bottom, unsigned int predIdx) const;
+
+  virtual double SetNode(unsigned int sIdx, double val, unsigned int sCount, unsigned int ctg) = 0;
 
  public:
-  static class SampleCtg *FactoryCtg(const class PMTrain *pmTrain, const std::vector<double> &y, const class RowRank *rowRank, const std::vector<unsigned int> &yCtg);
-  static class SampleReg *FactoryReg(const class PMTrain *pmTrain, const std::vector<double> &y, const class RowRank *rowRank, const std::vector<unsigned int> &row2Rank);
+  static class SampleCtg *FactoryCtg(const std::vector<double> &y, const class RowRank *rowRank, const std::vector<unsigned int> &yCtg, unsigned int _nCtg);
+  static class SampleReg *FactoryReg(const std::vector<double> &y, const class RowRank *rowRank, const std::vector<unsigned int> &row2Rank);
+  virtual class SplitPred *SplitPredFactory(const PMTrain *pmTrain, const RowRank *rowRank, const class Coproc *coproc) const = 0;
 
-  static void Immutables(unsigned int _nSamp, const std::vector<double> &_feSampleWeight, bool _withRepl, unsigned int _ctgWidth, unsigned int _nTree);
+  static void Immutables(unsigned int _nSamp, const std::vector<double> &_feSampleWeight, bool _withRepl);
   static void DeImmutables();
+  Sample(unsigned int _nRow, unsigned int nCtg);
+  virtual ~Sample();
+
+  void Stage(const class RowRank *rowRank, class SamplePred *samplePred, class Bottom *bottom) const;
+  class IndexLevel *IndexFactory(const class PMTrain *pmTrain, const class RowRank *rowRank, const class Coproc *coproc) const;
+
 
-  Sample();
   void RowInvert(std::vector<unsigned int> &sample2Row) const;
+
   
   /**
      @brief Accessor for sample count.
@@ -121,11 +172,6 @@ class Sample {
   }
 
 
-  const std::vector<SampleNode> &StageSample() {
-    return sampleNode;
-  }    
-
-  
   /**
      @param row row index at which to look up sample index.
 
@@ -150,16 +196,6 @@ class Sample {
   }
 
   
-  inline class Bottom *Bot() {
-    return bottom;
-  }
-
-  
-  inline class SamplePred *SmpPred() {
-    return samplePred;
-  }
-
-    
   inline const class BV *TreeBag() const {
     return treeBag;
   }
@@ -178,9 +214,6 @@ class Sample {
   inline FltVal Sum(int sIdx) const {
     return sampleNode[sIdx].Sum();
   }
-
-  
-  virtual ~Sample();
 };
 
 
@@ -190,16 +223,28 @@ class Sample {
 class SampleReg : public Sample {
   unsigned int *sample2Rank; // Only client currently leaf-based methods.
   void SetRank(const std::vector<unsigned int> &row2Rank);
- public:
-  SampleReg();
-  ~SampleReg();
 
+ public:
+  SampleReg(unsigned int _nRow);
+  ~SampleReg();
+  SplitPred *SplitPredFactory(const PMTrain *pmTrain, const RowRank *rowRank, const class Coproc *coproc) const;
+
+  
+  inline double SetNode(unsigned int sIdx, double yVal, unsigned int sCount, unsigned int ctg) {
+    SampleNode sNode;
+    double ySum = sNode.Set(yVal, sCount);
+    sampleNode[sIdx] = sNode;
+
+    return ySum;
+  }
+  
+  
   inline unsigned int Rank(unsigned int sIdx) const {
     return sample2Rank[sIdx];
   }
 
 
-  void Stage(const class PMTrain *pmTrain, const std::vector<double> &y, const std::vector<unsigned int> &row2Rank, const class RowRank *rowRank);
+  void PreStage(const std::vector<double> &y, const std::vector<unsigned int> &row2Rank, const class RowRank *rowRank);
 };
 
 
@@ -207,15 +252,26 @@ class SampleReg : public Sample {
  @brief Classification-specific sampling.
 */
 class SampleCtg : public Sample {
-  static unsigned int ctgWidth;
- public:
-  SampleCtg();
-  ~SampleCtg();
-  static void Immutables(unsigned int _ctgWidth, unsigned int _nTree);
-  static void DeImmutables();
+  const unsigned int nCtg;
 
+
+ public:
+  SampleCtg(unsigned int _nRow, unsigned int _nCtg);
+  ~SampleCtg();
+
+  SplitPred *SplitPredFactory(const PMTrain *pmTrain, const RowRank *rowRank, const class Coproc *coproc) const;
+
+  inline double SetNode(unsigned int sIdx, double yVal, unsigned int sCount, unsigned int ctg) {
+    SampleNode sNode;
+    double ySum = sNode.Set(yVal, sCount, ctg);
+    sampleNode[sIdx] = sNode;
+    ctgRoot[ctg].Accum(ySum, sCount);
+
+    return ySum;
+  }
   
-  void Stage(const class PMTrain *pmTrain, const std::vector<unsigned int> &yCtg, const std::vector<double> &y, const class RowRank *rowRank);
+  
+  void PreStage(const std::vector<unsigned int> &yCtg, const std::vector<double> &y, const class RowRank *rowRank);
 };
 
 
